#!/bin/sh
# All rights reserved

# Description: for every zone in the PowerDNS database, the scripts dumps DNS records

# --------------------------------------------------------------------------------
#
# Please modify the default DNS records target (backup file)
# You must include the full path!
# The program can also take a dump destination as the argument
config="/etc/pdns/pdns.conf"

#The mysql binary location 
my_bin="/usr/bin/mysql"

# NOTE that if 1 argument is given dump_file will be the argument
dump_file="/usr/local/full_recs.txt"
dump_temp_file="/tmp/dump_file-$$"

# Dump type can be <masters/slaves/both>
dump="masters"

# Converts master zones to slaves in dump (for DNS Cloud 3 acting as slave for a bind server)
# Works only when
# dump=masters/both
# dump_allow_transfers=yes/no
# dump_masters=yes/no
masters2slaves="no"

# Dump 'master' records from existing zones and includes them in the dump
# Works only when:
# dump=slaves/both
# masters2slaves=yes/no (Zones transformed from master2slave can not have a
# masters record because it does not exist in database!)
# dump_allow_transfers=yes/no
dump_masters="yes"

# Dump 'allow-transfer' servers from existing zones and includes them in the dump (CURRENTLY NOT USED)
# Works only when:
# dump=master/both
# dump_masters=yes/no
# masters2slaves=yes/no
dump_allow_transfers="yes"

# Dump reverse zones
dump_reverse="yes"

# Dump soa records only valid when exporting from DNS Manageger >3.0.0
dump_soa="yes"

#Whether to export or not disabled zones or records from DNS Manageger >3.0.0
#export_disabled="no"

# DO NOT MODIFY ANYTHING BELOW THIS LINE UNLESS YOU KNOW WHAT YOU ARE DOING
# -------------------------------------------------------------------------

is_root()
{
	current_user="`id`"
	case ${current_user} in
		uid=0\(root\)*)
				;;
			    *)
			    	echo "$0: You must have root privileges to run this script"
				echo "Su as root and try again"
				echo
				exit 1
				;;
	esac
}

get_config()
{
	if [ ! -f "$config" ];then
		echo '====> Config file does not exist. Trying to determine dnsmanager config '
		exit 1
	fi
	if [ ! -e "$my_bin" ]; then
		echo '====> Mysql binary does not exist. Please corect my_bin setting.'
		exit 1
	fi
	db_name="`cat $config | grep gmysql-dbname |awk '{split($1,a,"="); print a[2]}'`"
	    if [ -z $db_name ];then
	    	    echo "Unable to find DNS Cloud database name."
		    exit 1
    	    fi
        db_usr="`cat $config | grep gmysql-user |awk '{split($1,a,"="); print a[2]}'`"
	    if [ -z $db_usr ];then
		    echo "Unable to find DNS Cloud database user."
		    exit 1
	    fi
	db_passwd="`cat $config | grep gmysql-password |awk '{split($1,a,"="); print a[2]}'`"
	    if [ -z $db_passwd ];then
		echo "Unable to find DNS Cloud database password."
		exit 1
	    fi
}

get_masters()
{
	if [ "$masters2slaves" == "yes" ];then
		local mastertype="slave"
	else
		local mastertype="master"
	fi
	if [ "$dump_reverse" == "yes" ];then
		local domain_ids=`$my_bin -u$db_usr -p$db_passwd -Nse'select id from domains where type="MASTER";' $db_name |awk '{print $1}'`
	else
		local domain_ids=`$my_bin -u$db_usr -p$db_passwd -Nse'select id from domains where type="MASTER" and name not like "%.in-addr.arpa";' $db_name |awk '{print $1}'`
	fi
	echo > $dump_temp_file
	for i in $domain_ids; do
		$my_bin -u$db_usr -p$db_passwd -Nse"select name from domains where id=$i;" $db_name |awk -v type=$mastertype '{printf("%s.|%s {\n", $1,type)}' >> $dump_temp_file
		if [ "$masters2slaves" == "no" ];then
			$my_bin -u$db_usr -p$db_passwd -Nse"select type,name,content,prio from records where domain_id=$i;" $db_name | awk '{
			if (toupper($1)=="PTR"){
				split($2,ip,".");
				printf ("|%s| |%s.%s.%s.%s.in-addr.arpa.| |%s| |%s|\n",$1,ip[4],ip[3],ip[2],ip[1],$3,24);
			}else {
				if (toupper($1)=="MX") printf ("|%s| |%s| |%s| |%s|\n",$1,$2,$3,$4);
				if (toupper($1)!="TXT" || toupper($1)!="MX") printf ("|%s| |%s| |%s| ||\n",$1,$2,$3);
				else { type=$1;host=$2;str=$0;sub($1,"",str);
					if ($NF == "0" )sub($NF,"",str);
					sub($2,"",str);
					gsub(/^[ \t]+|[ \t]+$/,"",str);
					printf ("|%s| |%s| |%s| |0|\n",type,host,str);
				}
			}
			}' >> $dump_temp_file
#			if [ "$dump_allow_transfers" == "yes" ];then
#				$my_bin -u$db_usr -p$db_passwd -Nse"select master from domains where id=$i and type='SLAVE'" $db_name |awk '{printf ("|%s| |%s| || ||\n",toupper($1),$2)}' >> $dump_temp_file
#			fi
		fi
		printf "}\n" >> $dump_temp_file
		index=`expr $index + 1`
	done
	return $index;
}

get_slaves()
{
	if [ "$dump_reverse" == "yes" ];then
		local domain_ids=`$my_bin -u$db_usr -p$db_passwd -Nse'select id from domains where type="SLAVE";' $db_name |awk '{print $1}'`
	else
		local domain_ids=`$my_bin -u$db_usr -p$db_passwd -Nse'select id from domains where type="SLAVE" and name not like "%.in-addr.arpa";' $db_name |awk '{print $1}'`
	fi

	for i in $domain_ids; do
		$my_bin -u$db_usr -p$db_passwd -Nse"select name from domains where id=$i;" $db_name|awk '{printf("%s.|slave {\n",$1)}' >>$dump_temp_file
		if [ "$dump_masters" == "yes" ];then
			$my_bin -u$db_usr -p$db_passwd -Nse "select master from domains where id=$i;" $db_name|awk '{printf("|%s| |%s| || ||\n",toupper($1),$2)}' >>$dump_temp_file
		fi
		printf "}\n" >> $dump_temp_file
		index=`expr $index + 1`
	done
}


# main
is_root
get_config

if [ $# -eq 1 ]; then
    printf "Using $1 as a dump file\n"
    dump_file=$1
    dump_temp_file="$dump_file-$$"
fi

if [ $# -gt 1 ]; then
    exit 1
fi


index=0
echo -n > $dump_temp_file
case "$dump" in
            masters)
                get_masters
    	    ;;
    	    slaves)
                get_slaves
	    ;;
    	    both)
                get_masters
    	        get_slaves
	    ;;
esac
mv $dump_temp_file $dump_file >>/dev/null 2>&1
echo "$index complete zones dumped succesfully!"

exit 0
